require 'active_support'
require 'active_support/core_ext'

module FuckBot
  class MessageResolver
    BAD_WORD_RESPONSE = "Watch your language, <@%s>! «%s» is a bad word!".freeze

    def initialize(data, client)
      @data = data
      @client = client
    end

    def bad
      found_bad_word = find_bad_word

      return unless found_bad_word

      @client.store.increment(@data['user']).save

      send_response(
        format(BAD_WORD_RESPONSE, @data['user'], found_bad_word)
      )
    end

    def stats
      return unless stats?

      send_response(@client.store.to_stats)
    end

    def ping
      return unless ping?

      send_response(:pong)
    end

    def call
      return unless text_message? && !self?

      rules.each do |rule|
        break if public_send(rule)
      end
    end

    private

      def rules
        self.class.instance_methods(false) - [:call]
      end

      def text_message?
        @data['type'] == 'message' && !@data['text'].to_s.empty?
      end

      def stats?
        @data['text'] == "<@#{@client.user['id']}>: stats"
      end

      def ping?
        @data['text'] == "<@#{@client.user['id']}>: ping"
      end

      def self?
        @data['user'] == @client.user['id']
      end

      def find_bad_word
        text = @data['text'].mb_chars.downcase.to_s

        FuckBot::Dictionary.instance.words.find do |word|
          text.include?(word)
        end
      end

      def send_response(text)
        @client.ws.send(
          Oj.dump(
            id: generate_id,
            type: :message,
            channel: @data['channel'],
            text: text
          )
        )
      end

      def generate_id
        (Time.now.to_f * 100_000).to_i
      end
  end
end
